var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { html, svg } from "lit";
import { property, queryAll, state } from "lit/decorators.js";
import { ERROR_MESSAGE } from "../base/constant";
import { visiblePropConverter } from "../base/converter";
import { BaseError } from "../base/error";
import { createStyleOnHeader, dispatchCustomEvent, generateGUID, KucBase, } from "../base/kuc-base";
import { BaseLabel } from "../base/label";
import { validateArrayType, validateProps } from "../base/validator";
import { CHECKBOX_CSS } from "./style";
export { BaseError, BaseLabel };
let exportCheckbox;
(() => {
    exportCheckbox = window.customElements.get("kuc-checkbox-1-21-0");
    if (exportCheckbox) {
        return;
    }
    class KucCheckbox extends KucBase {
        constructor(props) {
            super();
            this.className = "";
            this.error = "";
            this.id = "";
            this.itemLayout = "horizontal";
            this.label = "";
            this.borderVisible = true;
            this.disabled = false;
            this.requiredIcon = false;
            this.visible = true;
            this.items = [];
            this.selectedIndex = [];
            this.value = [];
            this._valueMapping = {};
            this._GUID = generateGUID();
            const validProps = validateProps(props);
            this._setInitialValue(validProps);
            Object.assign(this, validProps);
        }
        _setInitialValue(validProps) {
            const hasValue = "value" in validProps;
            const hasSelectedIndex = "selectedIndex" in validProps;
            const _selectedIndex = validProps.selectedIndex || [];
            if (!hasValue && hasSelectedIndex) {
                if (!validateArrayType(_selectedIndex))
                    return;
                const _valueMapping = this._getValueMapping(validProps);
                this.value = this._getValidValue(_valueMapping, _selectedIndex);
            }
        }
        shouldUpdate(changedProperties) {
            if (changedProperties.has("items")) {
                if (!validateArrayType(this.items)) {
                    this.throwErrorAfterUpdateComplete(ERROR_MESSAGE.ITEMS.IS_NOT_ARRAY);
                    return false;
                }
            }
            if (changedProperties.has("value")) {
                if (!validateArrayType(this.value)) {
                    this.throwErrorAfterUpdateComplete(ERROR_MESSAGE.VALUE.IS_NOT_ARRAY);
                    return false;
                }
            }
            if (changedProperties.has("selectedIndex")) {
                if (!validateArrayType(this.selectedIndex)) {
                    this.throwErrorAfterUpdateComplete(ERROR_MESSAGE.SELECTED_INDEX.IS_NOT_ARRAY);
                    return false;
                }
            }
            return true;
        }
        willUpdate(changedProperties) {
            if (changedProperties.has("value")) {
                if (this.value.length > 0)
                    return;
                this.selectedIndex = [];
            }
        }
        _getNewValueMapping(value, selectedIndex) {
            const selectedIndexNumber = parseInt(selectedIndex, 10);
            const keys = Object.keys(this._valueMapping);
            const newValue = { ...this._valueMapping };
            if (keys.indexOf(selectedIndex) > -1) {
                delete newValue[selectedIndexNumber];
                return newValue;
            }
            newValue[selectedIndexNumber] = value;
            return newValue;
        }
        _handleChangeInput(event) {
            event.stopPropagation();
            const inputEl = event.target;
            const selectedIndex = inputEl.dataset.index || "0";
            const value = inputEl.value;
            const oldValue = !this.value ? this.value : [...this.value];
            const newValueMapping = this._getNewValueMapping(value, selectedIndex);
            const itemsValue = this.items.map((item) => item.value);
            const newValue = Object.values(newValueMapping).filter((item) => itemsValue.indexOf(item) > -1);
            if (newValue === oldValue)
                return;
            const newSelectedIndex = Object.keys(newValueMapping).map((item) => parseInt(item, 10));
            this.value = newValue;
            this.selectedIndex = newSelectedIndex;
            const eventDetail = {
                oldValue,
                value: newValue,
            };
            dispatchCustomEvent(this, "change", eventDetail);
        }
        _handleFocusInput(event) {
            const inputEl = event.target;
            const menuEl = inputEl.parentNode;
            menuEl.setAttribute("focused", "");
        }
        _handleBlurInput(event) {
            const inputEl = event.target;
            const menuEl = inputEl.parentNode;
            menuEl.removeAttribute("focused");
        }
        _getCheckboxIconSvgTemplate(disabled, checked) {
            return svg `
    <svg
      class="kuc-checkbox-1-21-0__group__select-menu__item__label__icon"
      width="21"
      height="21"
      viewBox="0 0 21 21"
      fill="none"
      xmlns="http://www.w3.org/2000/svg"
    >
      <rect
        x="1"
        y="1"
        width="19"
        height="19"
        rx="1"
        fill="white"
        stroke="${this._getSVGStrokeValue(disabled, checked)}"
        stroke-width="2"/>
      ${checked
                ? svg `<path
            fill-rule="evenodd"
            clip-rule="evenodd"
            d="M5 11L6.5 9L9.5 11.5L14.5 6L16 7.5L9.5 14.5L5 11Z"
            fill="${disabled ? "#d8d8d8" : "#3498db"}"/>`
                : ""}
    </svg>
  `;
        }
        _getSVGStrokeValue(disabled, checked) {
            if (disabled)
                return "#d8d8d8";
            if (checked)
                return "#3498db";
            return "#d8d8d8";
        }
        _isCheckedItem(item, index) {
            const values = Object.values(this._valueMapping);
            const keys = Object.keys(this._valueMapping);
            const result = values.filter((val, indexVal) => val === item.value && index === parseInt(keys[indexVal], 10));
            return result.length > 0;
        }
        _getItemTemplate(item, index) {
            const isCheckedItem = this._isCheckedItem(item, index);
            const isDisabled = item.disabled || this.disabled;
            return html `
        <div
          class="kuc-checkbox-1-21-0__group__select-menu__item"
          itemLayout="${this.itemLayout}"
        >
          <input
            type="checkbox"
            aria-describedby="${this._GUID}-error"
            aria-required="${this.requiredIcon}"
            data-index="${index}"
            id="${this._GUID}-item-${index}"
            class="kuc-checkbox-1-21-0__group__select-menu__item__input"
            name="${this._GUID}-group"
            value="${item.value !== undefined ? item.value : ""}"
            ?disabled="${isDisabled}"
            @change="${this._handleChangeInput}"
            @focus="${this._handleFocusInput}"
            @blur="${this._handleBlurInput}"
          />
          <label
            for="${this._GUID}-item-${index}"
            class="kuc-checkbox-1-21-0__group__select-menu__item__label"
            >${this._getCheckboxIconSvgTemplate(isDisabled, isCheckedItem)}${item.label === undefined ? item.value : item.label}
          </label>
        </div>
      `;
        }
        update(changedProperties) {
            if (changedProperties.has("items") ||
                changedProperties.has("value") ||
                changedProperties.has("selectedIndex")) {
                this._valueMapping = this._getValueMapping({
                    items: this.items,
                    value: this.value,
                    selectedIndex: this.selectedIndex,
                });
                this._setValueAndSelectedIndex();
            }
            super.update(changedProperties);
        }
        render() {
            return html `
        <div
          class="kuc-checkbox-1-21-0__group"
          role="group"
          aria-labelledby="${this._GUID}-group"
        >
          <div class="kuc-checkbox-1-21-0__group__label" ?hidden="${!this.label}">
            <kuc-base-label-1-21-0
              .text="${this.label}"
              .guid="${this._GUID}"
              .requiredIcon="${this.requiredIcon}"
            ></kuc-base-label-1-21-0>
          </div>
          <div
            class="kuc-checkbox-1-21-0__group__select-menu"
            ?borderVisible="${this.borderVisible}"
            itemLayout="${this.itemLayout}"
          >
            ${this.items.map((item, index) => this._getItemTemplate(item, index))}
          </div>
          <kuc-base-error-1-21-0
            .text="${this.error}"
            .guid="${this._GUID}"
            ?hidden="${!this.error}"
            ariaLive="assertive"
          ></kuc-base-error-1-21-0>
        </div>
      `;
        }
        updated() {
            this._inputEls.forEach((inputEl) => {
                inputEl.checked = this.value.indexOf(inputEl.value) > -1;
            });
        }
        _getValueMapping(validProps) {
            const _items = validProps.items || [];
            const _value = validProps.value || [];
            const _selectedIndex = validProps.selectedIndex || [];
            const itemsValue = _items.map((item) => item.value || "");
            const itemsMapping = Object.assign({}, itemsValue);
            const result = {};
            if (_value.length === 0) {
                const value = this._getValidValue(itemsMapping, _selectedIndex);
                _selectedIndex.forEach((key, i) => (result[key] = value[i]));
                return result;
            }
            const validSelectedIndex = this._getValidSelectedIndex(itemsMapping);
            validSelectedIndex.forEach((key, i) => (result[key] = _value[i]));
            return result;
        }
        _getValidValue(itemsMapping, _selectedIndex) {
            return _selectedIndex
                .filter((item) => itemsMapping[item])
                .map((item) => itemsMapping[item]);
        }
        _getValidSelectedIndex(itemsMapping) {
            const validSelectedIndex = [];
            for (let i = 0; i < this.value.length; i++) {
                const selectedIndex = this.selectedIndex[i];
                if (itemsMapping[selectedIndex] === this.value[i]) {
                    validSelectedIndex.push(selectedIndex);
                    continue;
                }
                const firstIndex = this.items.findIndex((item) => item.value === this.value[i]);
                validSelectedIndex.push(firstIndex);
            }
            return validSelectedIndex;
        }
        _setValueAndSelectedIndex() {
            this.value = Object.values(this._valueMapping);
            this.selectedIndex = Object.keys(this._valueMapping).map((key) => parseInt(key, 10));
        }
    }
    __decorate([
        property({ type: String, reflect: true, attribute: "class" })
    ], KucCheckbox.prototype, "className", void 0);
    __decorate([
        property({ type: String })
    ], KucCheckbox.prototype, "error", void 0);
    __decorate([
        property({ type: String, reflect: true, attribute: "id" })
    ], KucCheckbox.prototype, "id", void 0);
    __decorate([
        property({ type: String })
    ], KucCheckbox.prototype, "itemLayout", void 0);
    __decorate([
        property({ type: String })
    ], KucCheckbox.prototype, "label", void 0);
    __decorate([
        property({ type: Boolean })
    ], KucCheckbox.prototype, "borderVisible", void 0);
    __decorate([
        property({ type: Boolean })
    ], KucCheckbox.prototype, "disabled", void 0);
    __decorate([
        property({ type: Boolean })
    ], KucCheckbox.prototype, "requiredIcon", void 0);
    __decorate([
        property({
            type: Boolean,
            attribute: "hidden",
            reflect: true,
            converter: visiblePropConverter,
        })
    ], KucCheckbox.prototype, "visible", void 0);
    __decorate([
        property({ type: Array })
    ], KucCheckbox.prototype, "items", void 0);
    __decorate([
        property({ type: Array })
    ], KucCheckbox.prototype, "selectedIndex", void 0);
    __decorate([
        property({ type: Array })
    ], KucCheckbox.prototype, "value", void 0);
    __decorate([
        queryAll(".kuc-checkbox-1-21-0__group__select-menu__item__input")
    ], KucCheckbox.prototype, "_inputEls", void 0);
    __decorate([
        state()
    ], KucCheckbox.prototype, "_valueMapping", void 0);
    window.customElements.define("kuc-checkbox-1-21-0", KucCheckbox);
    createStyleOnHeader(CHECKBOX_CSS);
    exportCheckbox = KucCheckbox;
})();
const Checkbox = exportCheckbox;
export { Checkbox };
