var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { html, svg } from "lit";
import { property, query, queryAll, state } from "lit/decorators.js";
import { ERROR_MESSAGE } from "../base/constant";
import { unsafeHTMLConverter, visiblePropConverter } from "../base/converter";
import { createStyleOnHeader, dispatchCustomEvent, generateGUID, KucBase, } from "../base/kuc-base";
import { validateArrayType, validateDuplicatedValues, validateProps, validateValueString, } from "../base/validator";
import { TABS_CSS } from "./style";
let exportTabs;
(() => {
    exportTabs = window.customElements.get("kuc-tabs-1-21-0");
    if (exportTabs) {
        return;
    }
    const SCROLL_POSITION_THRESHOLD = 2; // pixels
    class KucTabs extends KucBase {
        constructor(props) {
            super();
            this.className = "";
            this.id = "";
            this.value = "";
            this.borderVisible = true;
            this.scrollButtons = false;
            this.visible = true;
            this.items = [];
            this._selectedValue = "";
            this._resizeObserver = null;
            this._isClick = false;
            this._isAtStart = true;
            this._isAtEnd = false;
            this._GUID = generateGUID();
            const validProps = validateProps(props);
            Object.assign(this, validProps);
            this._handleResize = this._handleResize.bind(this);
        }
        shouldUpdate(changedProperties) {
            if (changedProperties.has("items")) {
                if (!validateArrayType(this.items)) {
                    this.throwErrorAfterUpdateComplete(ERROR_MESSAGE.ITEMS.IS_NOT_ARRAY);
                    return false;
                }
                for (let index = 0; index < this.items.length; index++) {
                    const itemValue = this.items[index].value;
                    if (itemValue === undefined) {
                        this.throwErrorAfterUpdateComplete(ERROR_MESSAGE.ITEMS.IS_NOT_SPECIFIED);
                        return false;
                    }
                }
                const valueArray = this.items.map((item) => item.value);
                if (!validateDuplicatedValues(valueArray)) {
                    this.throwErrorAfterUpdateComplete(ERROR_MESSAGE.ITEMS.IS_DUPLICATED);
                    return false;
                }
            }
            if (changedProperties.has("value") && !validateValueString(this.value)) {
                this.throwErrorAfterUpdateComplete(ERROR_MESSAGE.VALUE.IS_NOT_STRING);
                return false;
            }
            return true;
        }
        willUpdate(changedProperties) {
            let tabIndex = this._getMatchedTabIndex();
            if (tabIndex === -1) {
                tabIndex = this._getFirstVisibleTabIndex();
            }
            tabIndex > -1 && (this._selectedValue = this.items[tabIndex].value);
        }
        render() {
            return html `
        <div class="kuc-tabs-1-21-0__group">
          <div class="kuc-tabs-1-21-0__group__tabs-container">
            <button
              class="kuc-tabs-1-21-0__group__tabs-container__tab-pre-button"
              @mousedown="${this._handleMouseDownPrevButton}"
              ?hidden="${!this.scrollButtons}"
              ?disabled="${this._isAtStart}"
              aria-hidden="true"
              tabindex="-1"
            >
              ${this._getPrevButtonSvgTemplate()}
            </button>
            <div
              class="kuc-tabs-1-21-0__group__tabs-container__tab-list-container"
              @scroll="${this._handleScroll}"
            >
              <ul
                class="kuc-tabs-1-21-0__group__tabs-container__tab-list-container__tab-list"
                role="tablist"
                @blur="${this._handleBlur}"
              >
                ${this.items.map((item, index) => this._getTabTemplate(item, index))}
              </ul>
            </div>
            <button
              class="kuc-tabs-1-21-0__group__tabs-container__tab-next-button"
              @mousedown="${this._handleMouseDownNextButton}"
              ?hidden="${!this.scrollButtons}"
              ?disabled="${this._isAtEnd}"
              aria-hidden="true"
              tabindex="-1"
            >
              ${this._getNextButtonSvgTemplate()}
            </button>
          </div>
          <div
            class="kuc-tabs-1-21-0__group__tab-panel"
            ?border-visible="${this.borderVisible}"
          >
            ${this.items.map((item, index) => this._getTabContentTemplate(item, index))}
          </div>
        </div>
      `;
        }
        firstUpdated() {
            window.addEventListener("resize", this._handleResize);
            this._resizeObserver = new ResizeObserver(() => {
                if (this.scrollButtons) {
                    this._updatePreNextButtonState();
                }
            });
            this._resizeObserver.observe(this._tabListContainer);
            this._setScrollStyles();
            this._scrollToSelectedTab(true);
        }
        updated(changedProperties) {
            if (changedProperties.has("scrollButtons")) {
                this._setScrollStyles();
            }
            if (this.scrollButtons) {
                this._updatePreNextButtonState();
            }
        }
        disconnectedCallback() {
            super.disconnectedCallback();
            window.removeEventListener("resize", this._handleResize);
            if (this._resizeObserver) {
                this._resizeObserver.disconnect();
                this._resizeObserver = null;
            }
        }
        _getTabTemplate(item, index) {
            const isSelected = item.value === this._selectedValue;
            return html `<li
        role="presentation"
        class="kuc-tabs-1-21-0__group__tabs-container__tab-list-container__tab-list__tab"
      >
        <button
          role="tab"
          ?hidden="${item.visible === false}"
          aria-selected="${isSelected}"
          tabindex="${isSelected && !item.disabled ? "0" : "-1"}"
          class="kuc-tabs-1-21-0__group__tabs-container__tab-list-container__tab-list__tab__button ${this
                ._isClick
                ? "kuc-tabs-1-21-0__group__tabs-container__tab-list-container__tab-list__tab__button--click"
                : ""}"
          id="${this._GUID}-button-${index}"
          aria-controls="${this._GUID}-tabpanel-${index}"
          value="${item.value}"
          @click="${this._handleClickTab}"
          @mousedown="${this._handleMouseDown}"
          @keydown="${this._handleKeyDownTab}"
          ?disabled="${item.disabled}"
        >
          ${item.label ? item.label : ""}
        </button>
      </li>`;
        }
        _getTabContentTemplate(item, index) {
            const isSelected = item.value === this._selectedValue;
            return html `<div
        class="kuc-tabs-1-21-0__group__tab-panel__content"
        role="tabpanel"
        id="${this._GUID}-tabpanel-${index}"
        aria-labelledby="${this._GUID}-button-${index}"
        ?hidden="${!isSelected || item.visible === false}"
        @change="${this._handleChangeEvent}"
      >
        ${item.content ? unsafeHTMLConverter(item.content) : ""}
      </div>`;
        }
        _getPrevButtonSvgTemplate() {
            return svg `
        <svg
          width="9"
          height="15"
          viewBox="0 0 9 15"
          fill="none"
          xmlns="http://www.w3.org/2000/svg">
          <path
            fill-rule="evenodd"
            clip-rule="evenodd"
            d="M1.99061 7.5L9 0.0604158L7.06632 0L0 7.5L7.06632 15L9 14.9396L1.99061 7.5Z"
            fill="${this._isAtStart ? "GrayText" : "#333333"}"
          />
        </svg>
      `;
        }
        _getNextButtonSvgTemplate() {
            return svg `
      <svg
        width="9"
        height="15"
        viewBox="0 0 9 15"
        fill="none"
        xmlns="http://www.w3.org/2000/svg">
        <path
          fill-rule="evenodd"
          clip-rule="evenodd"
          d="M7.00939 7.5L0 0.0604158L1.93368 0L9 7.5L1.93368 15L0 14.9396L7.00939 7.5Z"
          fill="${this._isAtEnd ? "GrayText" : "#333333"}"
        />
      </svg>
      `;
        }
        _getVisibleTab(direction, currentTab) {
            const tabArray = Array.from(this._tabButtons);
            const currentIndex = tabArray.indexOf(currentTab);
            const increment = direction === "next" ? 1 : -1;
            const endIndex = direction === "next" ? tabArray.length : -1;
            for (let i = currentIndex + increment; i !== endIndex; i += increment) {
                if (!tabArray[i].hidden) {
                    return tabArray[i];
                }
            }
            return null;
        }
        _handleTabScroll(direction) {
            const targetTab = this._findVisibleTab(direction);
            if (!targetTab)
                return;
            const shouldScrollToFullyVisible = this._isTabPartiallyVisible(targetTab, direction);
            const tabToScroll = shouldScrollToFullyVisible
                ? targetTab
                : this._getVisibleTab(direction, targetTab);
            if (tabToScroll) {
                this._scrollTab(tabToScroll, {
                    direction,
                    mode: "edge",
                });
            }
            this._updatePreNextButtonState();
        }
        _findVisibleTab(direction) {
            const containerRect = this._tabListContainer.getBoundingClientRect();
            const tabArray = Array.from(this._tabButtons);
            const isTabVisible = (tab) => {
                const rect = tab.getBoundingClientRect();
                return !(rect.right <= containerRect.left || rect.left >= containerRect.right);
            };
            return direction === "next"
                ? tabArray.reverse().find(isTabVisible)
                : tabArray.find(isTabVisible);
        }
        _isTabPartiallyVisible(tab, direction) {
            const containerRect = this._tabListContainer.getBoundingClientRect();
            const tabRect = tab.getBoundingClientRect();
            return direction === "next"
                ? tabRect.right > containerRect.right + SCROLL_POSITION_THRESHOLD
                : tabRect.left < containerRect.left - SCROLL_POSITION_THRESHOLD;
        }
        _calculateScrollPosition(tab, options) {
            const { direction, mode } = options;
            const containerRect = this._tabListContainer.getBoundingClientRect();
            const tabRect = tab.getBoundingClientRect();
            let scrollLeft = this._tabListContainer.scrollLeft;
            if (mode === "edge" && direction) {
                scrollLeft +=
                    direction === "next"
                        ? tabRect.right - containerRect.right
                        : tabRect.left - containerRect.left;
            }
            else {
                const isTabOverflow = tabRect.width > containerRect.width;
                if (isTabOverflow && direction) {
                    scrollLeft +=
                        direction === "next"
                            ? tabRect.left - containerRect.left
                            : tabRect.right - containerRect.right;
                }
                else if (tabRect.left < containerRect.left) {
                    scrollLeft += tabRect.left - containerRect.left;
                }
                else if (tabRect.right > containerRect.right) {
                    scrollLeft += tabRect.right - containerRect.right;
                }
            }
            return Math.max(0, Math.min(scrollLeft, this._tabListContainer.scrollWidth -
                this._tabListContainer.clientWidth));
        }
        _scrollTab(tab, options) {
            const scrollLeft = this._calculateScrollPosition(tab, options);
            this._tabListContainer.scrollTo({
                left: scrollLeft,
                behavior: options.immediate ? "auto" : "smooth",
            });
        }
        _handleMouseDownPrevButton(event) {
            event.preventDefault();
            this._handleTabScroll("prev");
        }
        _handleMouseDownNextButton(event) {
            event.preventDefault();
            this._handleTabScroll("next");
        }
        _handleResize() {
            if (this.scrollButtons) {
                this._updatePreNextButtonState();
            }
        }
        _handleScroll() {
            this._updatePreNextButtonState();
        }
        _isScrollToLeft() {
            return this._tabListContainer.scrollLeft === 0;
        }
        _isScrollToRight() {
            const { scrollWidth, scrollLeft, clientWidth } = this._tabListContainer;
            return (Math.abs(scrollWidth - scrollLeft - clientWidth) <
                SCROLL_POSITION_THRESHOLD);
        }
        _setScrollStyles() {
            var _a;
            (_a = this._tabGroup.parentElement) === null || _a === void 0 ? void 0 : _a.style.setProperty("max-width", this.scrollButtons ? "100%" : "");
            this._tabListContainer.style.setProperty("overflow-x", this.scrollButtons ? "auto" : "visible");
        }
        _updatePreNextButtonState() {
            const isAtStart = this._isScrollToLeft();
            const isAtEnd = this._isScrollToRight();
            if (isAtStart !== this._isAtStart) {
                this._isAtStart = isAtStart;
            }
            if (isAtEnd !== this._isAtEnd) {
                this._isAtEnd = isAtEnd;
            }
        }
        _handleMouseDown(event) {
            this._isClick = true;
        }
        _handleClickTab(event) {
            const tabEl = event.target;
            tabEl.blur();
            const currentIndex = this._getCurrentTabIndex(tabEl.getAttribute("value"));
            this._tabButtons[currentIndex].focus();
            if (this.value === tabEl.value)
                return;
            const eventDetail = this._generateEventDetail(tabEl.value);
            dispatchCustomEvent(this, "change", eventDetail);
        }
        _handleChangeEvent(event) {
            event.stopPropagation();
        }
        _handleBlur(event) {
            this._isClick = false;
        }
        _handleKeyDownTab(event) {
            this._isClick = false;
            let doPreventEvent = false;
            switch (event.key) {
                case "Left":
                case "ArrowLeft": {
                    doPreventEvent = true;
                    this._moveToAdjacentTab(event.target, "prev");
                    break;
                }
                case "Right":
                case "ArrowRight": {
                    doPreventEvent = true;
                    this._moveToAdjacentTab(event.target, "next");
                    break;
                }
                case "Home": {
                    doPreventEvent = true;
                    this._moveToLastFirstTab(event.target, "first");
                    break;
                }
                case "End": {
                    doPreventEvent = true;
                    this._moveToLastFirstTab(event.target, "last");
                    break;
                }
            }
            if (doPreventEvent) {
                event.stopPropagation();
                event.preventDefault();
            }
        }
        _getCurrentTabIndex(currentValue) {
            let currentIndex = -1;
            for (let index = 0; index < this.items.length; index++) {
                if (this.items[index].value === currentValue) {
                    currentIndex = index;
                    break;
                }
            }
            if (currentIndex === -1) {
                currentIndex = this._getFirstVisibleTabIndex();
            }
            return currentIndex;
        }
        _getFirstVisibleTabIndex() {
            return this.items.findIndex((item) => item.visible !== false);
        }
        _getMatchedTabIndex() {
            return this.items.findIndex((item) => item.visible !== false && item.value === this.value);
        }
        _moveToLastFirstTab(triggerEl, direction) {
            const currentIndex = this._getCurrentTabIndex(triggerEl.getAttribute("value"));
            const increment = direction === "last" ? -1 : 1;
            let index = direction === "last" ? this.items.length - 1 : 0;
            while (index !== currentIndex) {
                if (this.items[index].visible !== false &&
                    this.items[index].disabled !== true) {
                    triggerEl.blur();
                    dispatchCustomEvent(this, "change", this._generateEventDetail(this._tabButtons[index].getAttribute("value")));
                    this._tabButtons[this._getCurrentTabIndex(this.value)].focus();
                    this._scrollTab(this._tabButtons[this._getCurrentTabIndex(this.value)], {
                        mode: "visible",
                    });
                    break;
                }
                index += increment;
            }
        }
        _moveToAdjacentTab(triggerEl, direction) {
            const currentIndex = this._getCurrentTabIndex(triggerEl.getAttribute("value"));
            const increment = direction === "next" ? 1 : -1;
            let index = currentIndex + increment;
            while (index !== currentIndex) {
                if (index === this.items.length) {
                    index = 0;
                }
                else if (index === -1) {
                    index = this.items.length - 1;
                }
                if (index === currentIndex) {
                    break;
                }
                if (this.items[index].visible !== false &&
                    this.items[index].disabled !== true) {
                    triggerEl.blur();
                    dispatchCustomEvent(this, "change", this._generateEventDetail(this._tabButtons[index].getAttribute("value")));
                    this._tabButtons[this._getCurrentTabIndex(this.value)].focus();
                    this._scrollTab(this._tabButtons[this._getCurrentTabIndex(this.value)], {
                        mode: "visible",
                        direction,
                    });
                    break;
                }
                index += increment;
            }
        }
        _scrollToSelectedTab(immediate = false) {
            if (!this.value || !this._tabButtons.length)
                return;
            const currentIndex = this._getCurrentTabIndex(this.value);
            if (currentIndex === -1)
                return;
            const targetTab = this._tabButtons[currentIndex];
            if (!targetTab)
                return;
            if (targetTab.hidden)
                return;
            this._scrollTab(targetTab, {
                mode: "visible",
                immediate,
            });
            this._updatePreNextButtonState();
        }
        _generateEventDetail(newValue) {
            const oldValue = this.value;
            this.value = newValue;
            const eventDetail = {
                oldValue,
                value: newValue,
            };
            return eventDetail;
        }
    }
    __decorate([
        property({ type: String, reflect: true, attribute: "class" })
    ], KucTabs.prototype, "className", void 0);
    __decorate([
        property({ type: String, reflect: true, attribute: "id" })
    ], KucTabs.prototype, "id", void 0);
    __decorate([
        property({ type: String })
    ], KucTabs.prototype, "value", void 0);
    __decorate([
        property({ type: Boolean })
    ], KucTabs.prototype, "borderVisible", void 0);
    __decorate([
        property({ type: Boolean })
    ], KucTabs.prototype, "scrollButtons", void 0);
    __decorate([
        property({
            type: Boolean,
            attribute: "hidden",
            reflect: true,
            converter: visiblePropConverter,
        })
    ], KucTabs.prototype, "visible", void 0);
    __decorate([
        property({ type: Array })
    ], KucTabs.prototype, "items", void 0);
    __decorate([
        queryAll(".kuc-tabs-1-21-0__group__tabs-container__tab-list-container__tab-list__tab__button")
    ], KucTabs.prototype, "_tabButtons", void 0);
    __decorate([
        query(".kuc-tabs-1-21-0__group__tabs-container__tab-list-container")
    ], KucTabs.prototype, "_tabListContainer", void 0);
    __decorate([
        query(".kuc-tabs-1-21-0__group")
    ], KucTabs.prototype, "_tabGroup", void 0);
    __decorate([
        state()
    ], KucTabs.prototype, "_isClick", void 0);
    __decorate([
        state()
    ], KucTabs.prototype, "_isAtStart", void 0);
    __decorate([
        state()
    ], KucTabs.prototype, "_isAtEnd", void 0);
    window.customElements.define("kuc-tabs-1-21-0", KucTabs);
    createStyleOnHeader(TABS_CSS);
    exportTabs = KucTabs;
})();
const Tabs = exportTabs;
export { Tabs };
